import 'package:json_annotation/json_annotation.dart';
import 'package:equatable/equatable.dart';
import 'package:snaxdart_models/v1.dart';

part 'app_state.g.dart';

@JsonSerializable()
class AppState extends Equatable {
  @JsonKey(name: 'wallet')
  final Wallet wallet;

  @JsonKey(name: 'accounts')
  final Accounts accounts;

  @JsonKey(name: 'global')
  final Global global;

  @JsonKey(name: 'transactions')
  final WalletTransactions transactions;

  @JsonKey(name: 'sign_up')
  final SignUp signUp;

  @JsonKey(name: 'transfer')
  final Transfer transfer;

  @JsonKey(name: 'view')
  final View view;

  AppState({
    this.wallet,
    this.accounts,
    this.global,
    this.transactions,
    view,
    transfer,
    signUp,
  })  : this.signUp = signUp ?? SignUp.initialState(),
        this.transfer = transfer ?? Transfer.initialState(),
        this.view = view ?? View.initialState(),
        super();

  AppState.initialState()
      : wallet = Wallet.initialState(),
        accounts = Accounts.initialState(),
        global = Global.initialState(),
        signUp = SignUp.initialState(),
        transfer = Transfer.initialState(),
        view = View.initialState(),
        transactions = WalletTransactions.initialState();

  AppState copyWith({
    Wallet wallet,
    Accounts accounts,
    Global global,
    SignUp signUp,
    WalletTransactions transactions,
    Transfer transfer,
    View view,
  }) {
    return AppState(
      wallet: wallet ?? this.wallet,
      accounts: accounts ?? this.accounts,
      global: global ?? this.global,
      signUp: signUp ?? this.signUp,
      transfer: transfer ?? this.transfer,
      transactions: transactions ?? this.transactions,
      view: view ?? this.view
    );
  }

  static AppState fromJson(dynamic json) {
    if (json == null) {
      return AppState.initialState();
    }

    return _$AppStateFromJson(json);
  }

  Map<String, dynamic> toJson() => _$AppStateToJson(this);

  @override
  String toString() => this.toJson().toString();
}
