import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';

part 'global.g.dart';

enum Platform { Twitter, Steem }

enum ApplicationRoute {
  Home,
  Settings,
  Help,
  TransferSearchUser,
  TransferSendTokens,
  More,
  SteemBinding,
  TwitterBinding,
  Backup
}

enum NetworkState { Up, Down, Unknown }

@JsonSerializable()
class Global extends Equatable {
  @JsonKey(name: 'show_wallet_was_created')
  final bool walletWasCreated;

  @JsonKey(name: 'show_wallet_was_imported')
  final bool walletWasImported;

  @JsonKey(name: 'has_backed_up_wallet')
  final bool hasBackedUpWallet;

  @JsonKey(name: 'has_run_before')
  final bool hasRunBefore;

  @JsonKey(name: 'loading')
  final bool loading;

  @JsonKey(name: 'last_screen')
  final ApplicationRoute lastRoute;

  @JsonKey(name: 'trust_fall', ignore: true)
  final bool trustFall;

  @JsonKey(name: 'trust_fall_warning_showed', ignore: true)
  final bool trustFallWarningShowed;

  @JsonKey(name: 'network_state')
  final NetworkState networkState;

  @JsonKey(name: 'version')
  final String version;

  Global(
      {walletWasCreated,
      walletWasImported,
      hasBackedUpWallet,
      hasRunBefore,
        loading,
      lastRoute,
      trustFall,
      trustFallWarningShowed,
      networkState,
      version})
      : this.walletWasCreated = walletWasCreated ?? false,
        this.walletWasImported = walletWasImported ?? false,
        this.hasBackedUpWallet = hasBackedUpWallet ?? false,
        this.hasRunBefore = hasRunBefore ?? false,
        this.loading = loading ?? false,
        this.lastRoute = lastRoute ?? null,
        this.trustFall = trustFall ?? null,
        this.trustFallWarningShowed = trustFallWarningShowed ?? false,
        this.networkState = networkState ?? NetworkState.Unknown,
        this.version = version ?? '1.1',
        super();

  Global.initialState()
      : walletWasCreated = false,
        walletWasImported = false,
        hasBackedUpWallet = false,
        hasRunBefore = false,
        loading = false,
        lastRoute = null,
        trustFall = null,
        trustFallWarningShowed = false,
        networkState = NetworkState.Unknown,
        version = '1.1';

  Global copyWith({
    bool walletWasCreated,
    bool walletWasImported,
    bool hasBackedUpWallet,
    bool hasRunBefore,
    bool loading,
    ApplicationRoute lastRoute,
    bool trustFall,
    bool trustFallWarningShowed,
    NetworkState networkState
  }) {
    return Global(
        walletWasCreated: walletWasCreated ?? this.walletWasCreated,
        walletWasImported: walletWasImported ?? this.walletWasImported,
        hasBackedUpWallet: hasBackedUpWallet ?? this.hasBackedUpWallet,
        hasRunBefore: hasRunBefore ?? this.hasRunBefore,
        loading: loading ?? this.loading,
        lastRoute: lastRoute ?? this.lastRoute,
        trustFall: trustFall ?? this.trustFall,
        trustFallWarningShowed:
            trustFallWarningShowed ?? this.trustFallWarningShowed,
        networkState: networkState ?? this.networkState);
  }

  factory Global.fromJson(Map<String, dynamic> json) => _$GlobalFromJson(json);

  Map<String, dynamic> toJson() => _$GlobalToJson(this);

  @override
  String toString() => this.toJson().toString();
}
