import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:snaxdart_models/v1.dart';
import 'package:snaxdart_errors/snaxdart_errors.dart';

part 'sign_up.g.dart';

@JsonSerializable()
class SignUp extends Equatable {
  @JsonKey(name: 'loading')
  final bool loading;

  @JsonKey(name: 'error')
  final SnaxError error;

  @JsonKey(name: 'snax_account')
  final String snaxAccount;

  @JsonKey(name: 'account')
  final WalletAccount account;

  @JsonKey(name: 'platform')
  final Platform platform;

  @JsonKey(name: 'hash')
  final String hash;

  @JsonKey(name: 'binding_salt')
  final String bindingSalt;

  @JsonKey(name: 'confirmed')
  final bool confirmed;

  @JsonKey(name: 'post_id')
  final String postId;

  SignUp(
      {this.loading,
      this.error,
      this.account,
      this.platform,
      this.snaxAccount,
      this.hash,
      this.bindingSalt,
      this.confirmed,
      this.postId})
      : super();

  SignUp.initialState()
      : loading = false,
        error = null,
        account = null,
        platform = null,
        hash = null,
        bindingSalt = null,
        snaxAccount = null,
        confirmed = null,
        postId = null;

  SignUp copyWith(
      {bool loading,
      SnaxError error,
      WalletAccount account,
      Platform platform,
      String hash,
      String bindingSalt,
      String snaxAccount,
      bool confirmed,
      String postId}) {
    return SignUp(
        loading: loading ?? this.loading,
        error: error is NoneError ? null : error ?? this.error,
        account: account ?? this.account,
        platform: platform ?? this.platform,
        hash: hash ?? this.hash,
        bindingSalt: bindingSalt ?? this.bindingSalt,
        snaxAccount: snaxAccount ?? this.snaxAccount,
        confirmed: confirmed ?? this.confirmed,
        postId: postId ?? this.postId);
  }

  SignUp reset() {
    return SignUp.initialState();
  }

  factory SignUp.fromJson(Map<String, dynamic> json) => _$SignUpFromJson(json);

  Map<String, dynamic> toJson() => _$SignUpToJson(this);

  @override
  String toString() => this.toJson().toString();
}
