import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';

part 'transaction.g.dart';

enum TransactionSource { Snax, Twitter, Steem }
enum TransactionType { INCOMING, OUTCOMING }

@JsonSerializable()
class WalletTransaction extends Equatable {
  @JsonKey(name: 'source')
  final TransactionSource source;

  @JsonKey(name: 'type')
  final TransactionType type;

  @JsonKey(name: 'block_time')
  final String blockTime;

  @JsonKey(name: 'block_number')
  final int blockNumber;

  @JsonKey(name: 'from')
  final String from;

  @JsonKey(name: 'from_account')
  final String fromAccount;

  @JsonKey(name: 'to')
  final String to;

  @JsonKey(name: 'quantity')
  final String quantity;

  @JsonKey(name: 'memo')
  final String memo;

  @JsonKey(name: 'global_sequence')
  final int globalSequence;

  @JsonKey(name: 'trx_id')
  final String trxId;

  WalletTransaction(
      {this.source,
      this.type,
      this.blockTime,
      this.blockNumber,
      this.from,
      this.fromAccount,
      this.to,
      this.quantity,
      this.memo,
      this.globalSequence,
      this.trxId})
      : super();

  factory WalletTransaction.fromJson(Map<String, dynamic> json) =>
      _$WalletTransactionFromJson(json);

  Map<String, dynamic> toJson() => _$WalletTransactionToJson(this);

  @override
  String toString() => this.toJson().toString();
}

@JsonSerializable()
class IncompleteWalletTransaction extends WalletTransaction {
  @JsonKey(name: 'source')
  final TransactionSource source;

  @JsonKey(name: 'type')
  final TransactionType type;

  @JsonKey(name: 'block_time')
  final String blockTime;

  @JsonKey(name: 'from')
  final String from;

  @JsonKey(name: 'from_account')
  final String fromAccount;

  @JsonKey(name: 'to')
  final String to;

  @JsonKey(name: 'quantity')
  final String quantity;

  @JsonKey(name: 'memo')
  final String memo;

  @JsonKey(name: 'trx_id')
  final String trxId;

  IncompleteWalletTransaction({
    this.source,
    this.type,
    this.blockTime,
    this.from,
    this.fromAccount,
    this.to,
    this.quantity,
    this.memo,
    this.trxId
  }) : super();

  factory IncompleteWalletTransaction.fromJson(Map<String, dynamic> json) =>
      _$IncompleteWalletTransactionFromJson(json);

  Map<String, dynamic> toJson() => _$IncompleteWalletTransactionToJson(this);

  @override
  String toString() => this.toJson().toString();
}
