import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:snaxdart_models/v1.dart';
import 'package:snaxdart_errors/snaxdart_errors.dart';

part 'transfer.g.dart';

@JsonSerializable()
class Transfer extends Equatable {
  @JsonKey(name: 'data')
  final WalletTransaction data;

  @JsonKey(name: 'loading')
  final bool loading;

  @JsonKey(name: 'error')
  final SnaxError error;

  @JsonKey(name: 'recipient')
  final String recipient;

  @JsonKey(name: 'platform')
  final TransactionSource platform;

  @JsonKey(name: 'memo')
  final String memo;

  @JsonKey(name: 'quantity')
  final String quantity;

  Transfer(
      {this.data,
      this.loading,
      this.error,
      this.recipient,
      this.platform,
      this.memo = '',
      this.quantity});

  Transfer.initialState()
      : data = null,
        loading = false,
        error = null,
        platform = TransactionSource.Twitter,
        recipient = null,
        memo = '',
        quantity = null;

  Transfer resetData() {
    return Transfer(
        data: null,
        loading: this.loading,
        error: null,
        platform: this.platform,
        recipient: this.recipient,
        memo: this.memo,
        quantity: this.quantity
    );
  }

  Transfer copyWith(
      {WalletTransaction data,
      bool loading,
      SnaxError error,
      TransactionSource platform,
      String recipient,
      String memo,
      String quantity}) {
    return Transfer(
        data: data ?? this.data,
        loading: loading ?? this.loading,
        error: error ?? this.error,
        platform: platform ?? this.platform,
        recipient: recipient ?? this.recipient,
        memo: memo ?? this.memo,
        quantity: quantity ?? this.quantity);
  }

  factory Transfer.fromJson(Map<String, dynamic> json) =>
      _$TransferFromJson(json);

  Map<String, dynamic> toJson() => _$TransferToJson(this);

  @override
  String toString() => this.toJson().toString();
}
