import 'package:test/test.dart';
import 'package:snaxdart_errors/snaxdart_errors.dart';

import 'package:snaxdart_models/v1.dart';
import 'package:snaxdart/snaxdart.dart';

void main() {
  group('Accounts Model', () {
    test('should create model with initial state', () {
      final accounts = Accounts.initialState();
      expect(accounts.data, equals([]));
      expect(accounts.loading, equals(true));
      expect(accounts.error, equals(null));
    });

    test('should create a copy with data', () {
      final initialAccounts = Accounts.initialState();

      final accounts = [
        WalletAccount(
          snax: Account('hellosnax', 1),
          twitter: TwitterAccount(id: '1', displayName: 'twitter'),
        )
      ];

      final newAccounts = initialAccounts.copyWith(data: accounts);

      expect(newAccounts.data.length, 1);
      expect(newAccounts.data.first.snax.accountName, 'hellosnax');
      expect(newAccounts.data.first.snax.headBlockNum, 1);
      expect(newAccounts.data.first.twitter.id, '1');
      expect(newAccounts.data.first.twitter.displayName, 'twitter');
      expect(newAccounts.loading, true);
      expect(newAccounts.error, null);
    });

    test('should create a copy with loading', () {
      final initialAccounts = Accounts.initialState();

      final accounts = initialAccounts.copyWith(loading: false);

      expect(accounts.data.isEmpty, true);
      expect(accounts.loading, false);
      expect(accounts.error, null);
    });
    test('should create a copy with error', () {
      final initialAccounts = Accounts.initialState();
      final error = new GeneralError('some bad error');

      final accounts = initialAccounts.copyWith(error: error);

      expect(accounts.data.isEmpty, true);
      expect(accounts.loading, true);
      expect(accounts.error, error);
    });
  });
}
