import 'package:snaxdart_models/v1.dart';
import 'package:snaxdart_errors/snaxdart_errors.dart';
import 'package:test/test.dart';

void main() {
  group('Transactions Model', () {
    test('should create model with initial state', () {
      final transactions = WalletTransactions.initialState();

      expect(transactions.data, []);
      expect(transactions.error, null);
      expect(transactions.loading, false);
      expect(transactions.hasMore, false);
    });
    test('should create copy with data', () {
      final transaction = WalletTransaction(
        source: TransactionSource.Snax,
        type: TransactionType.INCOMING,
        blockTime: '2019-04-29T14:38:10.000',
        blockNumber: 1,
        from: 'hellosnax',
        to: 'awesome',
        quantity: '100.0000 SNAX',
        memo: '',
        globalSequence: 1,
        trxId:
            "b94415c1g547988hff8v8ef054dade3sda5a0cd14dvbb3f542ge84a164c5da33",
      );

      final transactions =
          WalletTransactions.initialState().copyWith(data: [transaction]);

      expect(transactions.data.length, 1);
      expect(transactions.data.first.source, TransactionSource.Snax);
      expect(transactions.data.first.type, TransactionType.INCOMING);
      expect(transactions.data.first.blockTime, '2019-04-29T14:38:10.000');
      expect(transactions.data.first.blockNumber, 1);
      expect(transactions.data.first.from, 'hellosnax');
      expect(transactions.data.first.to, 'awesome');
      expect(transactions.data.first.quantity, '100.0000 SNAX');
      expect(transactions.data.first.memo, '');
      expect(transactions.data.first.globalSequence, 1);
      expect(transactions.data.first.trxId,
          'b94415c1g547988hff8v8ef054dade3sda5a0cd14dvbb3f542ge84a164c5da33');
      expect(transactions.error, null);
      expect(transactions.loading, false);
      expect(transactions.hasMore, false);
    });

    test('should create copy with loading', () {
      final initialTransactions = WalletTransactions.initialState();

      final transactions = initialTransactions.copyWith(loading: false);

      expect(transactions.loading, false);
    });

    test('should create copy with error', () {
      final initialTransactions = WalletTransactions.initialState();

      final transactions = initialTransactions.copyWith(
          error: new GeneralError('something wrong'));

      expect(transactions.error.message, ('something wrong'));
    });

    test('should create copy with hasMore', () {
      final initialTransactions = WalletTransactions.initialState();

      final transactions = initialTransactions.copyWith(hasMore: true);

      expect(transactions.hasMore, true);
    });

    test('should reset error', () {
      final initialTransactions = WalletTransactions.initialState()
          .copyWith(error: new GeneralError('Random error'));

      final transaction = initialTransactions.resetError();

      expect(transaction.error, null);
    });
  });
}
