import 'package:test/test.dart';

import 'package:snaxdart_models/v1.dart';
import 'package:snaxdart_errors/snaxdart_errors.dart';

void main() {
  group('Transfer Model', () {
    test('should create model with initial state', () {
      final transter = Transfer.initialState();

      expect(transter.data, null);
      expect(transter.error, null);
      expect(transter.loading, false);
    });
    test('should create copy with data', () {
      final transaction = WalletTransaction(
        source: TransactionSource.Snax,
        type: TransactionType.INCOMING,
        blockTime: '2019-04-29T14:38:10.000',
        blockNumber: 1,
        from: 'hellosnax',
        to: 'awesome',
        quantity: '100.0000 SNAX',
        memo: '',
        globalSequence: 1,
        trxId:
            "b94415c1g547988hff8v8ef054dade3sda5a0cd14dvbb3f542ge84a164c5da33",
      );

      final transter = Transfer.initialState().copyWith(data: transaction);

      expect(transter.data.source, TransactionSource.Snax);
      expect(transter.data.type, TransactionType.INCOMING);
      expect(transter.data.blockTime, '2019-04-29T14:38:10.000');
      expect(transter.data.blockNumber, 1);
      expect(transter.data.from, 'hellosnax');
      expect(transter.data.to, 'awesome');
      expect(transter.data.quantity, '100.0000 SNAX');
      expect(transter.data.memo, '');
      expect(transter.data.globalSequence, 1);
      expect(transter.data.trxId,
          'b94415c1g547988hff8v8ef054dade3sda5a0cd14dvbb3f542ge84a164c5da33');
      expect(transter.error, null);
      expect(transter.loading, false);
    });

    test('should create copy with loading', () {
      final initialTransfer = Transfer.initialState();

      final transfer = initialTransfer.copyWith(loading: true);

      expect(transfer.loading, true);
    });

    test('should create copy with error', () {
      final initialTransfer = Transfer.initialState();

      final error = new GeneralError('failed');

      final transfer = initialTransfer.copyWith(error: error);

      expect(transfer.error, error);
    });
  });
}
