import 'dart:io';
import 'package:test/test.dart';
import 'dart:convert';

import 'package:snaxdart_models/v1.dart';
import 'package:snaxdart_ecc/snaxdart_ecc.dart';
import 'package:snaxdart/snaxdart.dart';

void main() {
  group('Wallet Model', () {
    dynamic walletString;

    setUp(() {
      walletString = jsonDecode(
          File('./test/v1/test_data/wallet_test_data.json').readAsStringSync());
    });

    test('should create model with initial state', () {
      final wallet = Wallet.initialState();
      expect(wallet.account, null);
      expect(wallet.publicKey, null);
    });

    test('should create copy with publicKey', () {
      final initalWallet = Wallet.initialState();
      final publicKey =
          SNAXPrivateKey.fromRandom().toSNAXPublicKey().toString();

      final wallet = initalWallet.copyWith(publicKey: publicKey);

      expect(wallet.publicKey, publicKey);
      expect(wallet.account, null);
    });

    test('should create copy with account', () {
      final initalWallet = Wallet.initialState();
      final snaxAccount = Account.fromJson(walletString['account']);
      final twitterAccount =
          TwitterAccount(id: '5', displayName: 'SnaxTwitter');

      final account = WalletAccount(snax: snaxAccount, twitter: twitterAccount);

      final wallet = initalWallet.copyWith(account: account);

      expect(wallet.publicKey, null);
      expect(wallet.account, account);
      expect(wallet.account, account);
    });

    test('should create wallet from json', () {
      final wallet = Wallet.fromJson(walletString);

      expect(wallet.account.snax.accountName, 'binancecold1');
      expect(wallet.account.snax.coreLiquidBalance.toString(),
          '49254569.1575 SNAX');
      expect(wallet.publicKey,
          'SNAX78o85K6hKAkszNCo1J9njoGxoicFWdfYsAKdgUqCoQiQTPvfrD');
    });

    test('should create json from wallet', () {
      final wallet = Wallet(
        publicKey: 'SNAX78o85K6hKAkszNCo1J9njoGxoicFWdfYsAKdgUqCoQiQTPvfrD',
        account: null,
      );

      final json = wallet.toJson();
      final expectedJson = {
        'public_key': 'SNAX78o85K6hKAkszNCo1J9njoGxoicFWdfYsAKdgUqCoQiQTPvfrD',
        'account': null,
        'logged': false
      };

      expect(json, expectedJson);
    });
  });
}
